==> pywebsite-0.1.12pre.tar.gz: pywebsite-0.1.12pre/pywebsite/imageops/imageops.py <==
##    pywebsite - Python Website Library
##    Copyright (C) 2009 Rene Dudfield
##
##    This library is free software; you can redistribute it and/or
##    modify it under the terms of the GNU Library General Public
##    License as published by the Free Software Foundation; either
##    version 2 of the License, or (at your option) any later version.
##
##    This library is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##    Library General Public License for more details.
##
##    You should have received a copy of the GNU Library General Public
##    License along with this library; if not, write to the Free
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
##    Rene Dudfield
##    renesd@gmail.com
"""

purpose:
    Doing operations on images in a caching manner.
    Scaling, rotation, text overlays, watermarking etc.

eg, 
gallery=mypics&image=image001.jpg&rotate=90
gallery=mypics&image=image001.jpg&width=10&height=10



gallery
image
width
height
rotate



check for ok extensions, ie jpg, png etc.  Not .php or other executable files.


TODO: 
    - generate url.  Use a hash to make sure passed vars are valid.

"""

import os, stat
import pygame

# 64MiB
MAX_CACHE_DIR = 64 * (1024*1024)

class ImageOps(object):
    def __init__(self, 
                 cache_dir, 
                 path_to_galleries, 
                 cache_max_size = MAX_CACHE_DIR, 
                 max_image_size = (4096,4096)):
        """ 
            cache_dir - directory to store the cached images.
            cache_max_size - maximum size of the cache files in bytes.
            path_to_galleries - path to a directory of galleries.
            max_image_size - maximum size the cached images can be.
        """
        self.cache_dir = cache_dir
        self.path_to_galleries = path_to_galleries 
        self.cache_max_size = cache_max_size
        self.max_image_size = max_image_size

        # make sure the paths validate ok.  
        #       are they there?  are they writable?
        if not os.path.exists(cache_dir):
            raise ValueError("cache_dir :%s: does not exist" % cache_dir)
        #TODO: check if the cache dir is writable?  If not warning.
        
        if not os.path.exists(path_to_galleries):
            raise ValueError(("path_to_galleries :%s: does not exist" % 
                              path_to_galleries))



    def check_valid_paths(self, path_parts):
        ''' see if each part of the path is a safe, and valid path.
        '''
        if [p for p in path_parts if ".." in p]:
            return False
        if [p for p in path_parts if p.startswith('/')]:
            return False

        return True


    def check_size(self, width, height):
        '''
        '''
        mw, mh = self.max_image_size
        if width > mw or height > mh:
            return False

        return True



    def get_cached_fname(self, 
                         gallery,
                         image,
                         width,
                         height,
                         rotate = False):
        """
        """
        #"400x400-gallery-rotate-image.jpg"
        fmt = "%sx%s-%s-%s-%s"
        args = (width, height, gallery, rotate, image)
        fname = os.path.join(self.cache_dir, fmt % args)
        return fname


    def check_cached_fname(self, orig_fname, cached_fname):
        ''' returns False if cached file is not valid.
        '''
        if not os.path.exists(cached_fname):
            return False
        cached_time = os.stat(cached_fname)[stat.ST_MTIME]
        orig_time = os.stat(orig_fname)[stat.ST_MTIME]

        if cached_time < orig_time:
            # cached file is older than original file.
            return False

        return True

    def get_image_path(self, gallery, image):
        return os.path.join(self.path_to_galleries, gallery, image)

    def resize(self, 
               gallery,
               image,
               width,
               height,
               rotate = False):
        """ resize the image.  Return the path of the resized image.
        """
        width, height = int(width), int(height)

        if not self.check_valid_paths([image,gallery]):
            raise ValueError('invalid path')

        # generate the path.
        path = self.get_image_path(gallery, image)

        if not os.path.exists(path):
            raise IOError(2, "No such file or directory: '%s'" % path)

        if not self.check_size(width, height):
            raise ValueError('invalid image size')

        #print ("rotate:%s:" % rotate)
        if rotate in [0,False,'', '0']:
            rotate_float = False
        else:
            if type(rotate) != float:
                try:
                    if len(rotate) > 32:
                        raise ValueError("invalid rotate")
                except TypeError:
                        raise ValueError("invalid rotate")
                rotate_float = float(rotate)
            else:
                rotate_float = rotate

            if rotate_float < 0 or rotate_float > 360.0:
                raise ValueError('invalid rotation.  Must be between 0-360.')




        # if the cached file is already there, then we return it.
        cached_fname = self.get_cached_fname(gallery, image, width, height, 
                                             rotate_float)

        #TODO: check if it is a valid unix file name.
        
        
        # see if there is a valid cached version already there.
        if self.check_cached_fname(path, cached_fname):
            return cached_fname
        
        #TODO: check image size before loading it.
        if rotate_float is not False:
            raise NotImplementedError('rotate not implemented')
        else:
            surf = pygame.image.load(path)
            try:
                transformed_surf = pygame.transform.smoothscale(surf, 
                                                            (width, height))
            except ValueError: 
                transformed_surf = pygame.transform.scale(surf, 
                                                            (width, height))

        # TODO: save to a temporary file in the same directory, then move.
        pygame.image.save(transformed_surf, cached_fname)
        
        return cached_fname

==> pywebsite-0.1.14pre.tar.gz: pywebsite-0.1.14pre/pywebsite/imageops/imageops.py <==
##    pywebsite - Python Website Library
##    Copyright (C) 2009 Rene Dudfield
##
##    This library is free software; you can redistribute it and/or
##    modify it under the terms of the GNU Library General Public
##    License as published by the Free Software Foundation; either
##    version 2 of the License, or (at your option) any later version.
##
##    This library is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##    Library General Public License for more details.
##
##    You should have received a copy of the GNU Library General Public
##    License along with this library; if not, write to the Free
##    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##
##    Rene Dudfield
##    renesd@gmail.com
"""

purpose:
    Doing operations on images in a caching manner.
    Scaling, rotation, text overlays, watermarking etc.

eg, 
gallery=mypics&image=image001.jpg&rotate=90
gallery=mypics&image=image001.jpg&width=10&height=10


gallery
image
width
height
rotate



check for ok extensions, ie jpg, png etc.  Not .php or other executable files.

TODO: 
    - make imagemagick backend.

"""

import os, stat
import pygame

# 64MiB
MAX_CACHE_DIR = 64 * (1024*1024)

class ImageOps(object):
    def __init__(self, 
                 cache_dir, 
                 path_to_galleries, 
                 cache_max_size = MAX_CACHE_DIR, 
                 max_image_size = (4096,4096)):
        """ 
            cache_dir - directory to store the cached images.
            cache_max_size - maximum size of the cache files in bytes.
            path_to_galleries - path to a directory of galleries.
            max_image_size - maximum size the cached images can be.
        """
        self.cache_dir = cache_dir
        self.path_to_galleries = path_to_galleries 
        self.cache_max_size = cache_max_size
        self.max_image_size = max_image_size

        # make sure the paths validate ok.  
        #       are they there?  are they writable?
        if not os.path.exists(cache_dir):
            raise ValueError("cache_dir :%s: does not exist" % cache_dir)
        #TODO: check if the cache dir is writable?  If not warning.
        
        if not os.path.exists(path_to_galleries):
            raise ValueError(("path_to_galleries :%s: does not exist" % 
                              path_to_galleries))



    def check_valid_paths(self, path_parts):
        ''' see if each part of the path is a safe, and valid path.
        '''
        if [p for p in path_parts if ".." in p]:
            return False
        if [p for p in path_parts if p.startswith('/')]:
            return False

        return True


    def check_size(self, width, height):
        '''
        '''
        mw, mh = self.max_image_size
        if width > mw or height > mh:
            return False

        return True



    def get_cached_fname(self, 
                         gallery,
                         image,
                         width,
                         height,
                         rotate = False):
        """
        """
        #"400x400-gallery-rotate-image.jpg"
        fmt = "%sx%s-%s-%s-%s"
        args = (width, height, gallery, rotate, image)
        fname = os.path.join(self.cache_dir, fmt % args)
        return fname


    def check_cached_fname(self, orig_fname, cached_fname):
        ''' returns False if cached file is not valid.
        '''
        if not os.path.exists(cached_fname):
            return False
        cached_time = os.stat(cached_fname)[stat.ST_MTIME]
        orig_time = os.stat(orig_fname)[stat.ST_MTIME]

        if cached_time < orig_time:
            # cached file is older than original file.
            return False

        return True

    def get_image_path(self, gallery, image):
        return os.path.join(self.path_to_galleries, gallery, image)

    def resize(self, 
               gallery,
               image,
               width,
               height,
               rotate = False):
        """ resize the image.  Return the path of the resized image.
        """
        width, height = int(width), int(height)

        if not self.check_valid_paths([image,gallery]):
            raise ValueError('invalid path')

        # generate the path.
        path = self.get_image_path(gallery, image)

        if not os.path.exists(path):
            raise IOError(2, "No such file or directory: '%s'" % path)

        if not self.check_size(width, height):
            raise ValueError('invalid image size')

        #print ("rotate:%s:" % rotate)
        if rotate in [0,False,'', '0']:
            rotate_float = False
        else:
            if type(rotate) != float:
                try:
                    if len(rotate) > 32:
                        raise ValueError("invalid rotate")
                except TypeError:
                        raise ValueError("invalid rotate")
                rotate_float = float(rotate)
            else:
                rotate_float = rotate

            if rotate_float < 0 or rotate_float > 360.0:
                raise ValueError('invalid rotation.  Must be between 0-360.')




        # if the cached file is already there, then we return it.
        cached_fname = self.get_cached_fname(gallery, image, width, height, 
                                             rotate_float)

        #TODO: check if it is a valid unix file name.
        
        
        # see if there is a valid cached version already there.
        if self.check_cached_fname(path, cached_fname):
            return cached_fname
        
        #TODO: check image size before loading it.
        if rotate_float is not False:
            raise NotImplementedError('rotate not implemented')
        else:
            surf = pygame.image.load(path)
            try:
                transformed_surf = pygame.transform.smoothscale(surf, 
                                                            (width, height))
            except ValueError: 
                transformed_surf = pygame.transform.scale(surf, 
                                                            (width, height))

        # TODO: save to a temporary file in the same directory, then move.
        pygame.image.save(transformed_surf, cached_fname)
        
        return cached_fname


#class AspectSize(object):
#
#    def size(self, orig_width, orig_height, 
#aspect_width = 0, keep_aspect_height = 0):
#        """
#        >>> size(10,44)
#        """

def resize_percent(width, height, percent_width, percent_height):
    """ resize_percent(width, height, percent_width, percent_height)
        Returns a (width, height) scaled by the percentages given.
    >>> resize_percent(100, 100, 50, 50)
    (50, 50)
 
    >>> sizes = ((100, 100, 50, 50), (100, 100, 200, 50), 
    ...          (100, 100, 200, 0))
    >>> for s in sizes:
    ...     print (resize_percent(*s))
    (50, 50)
    (200, 50)
    (200, 0)
    """
    return int((percent_width*width)/100), int((percent_height*height)/100)


def resize_aspect(width, height, wanted_width = None, wanted_height = None):
    """ returns the (width, height)
    >>> resize_aspect(100, 120, None, 60)
    (50, 60)


    """
    # get the aspect ratio of the image.

    if wanted_width is None and wanted_height is None:
        raise ValueError("must give a wanted width and height")








if __name__ == "__main__":
    
    
    import doctest, sys
    if "test" in sys.argv:
        doctest.testmod()
